@@ -2,13 +2,14 @@
 # encoding: utf-8
 
 __author__ = "ChenyangGao <https://chenyanggao.github.io>"
-__version__ = (0, 2, 4)
+__version__ = (0, 2, 5)
 __all__ = [
     "Yield", "YieldFrom", "iterable", "async_iterable", 
     "run_gen_step", "run_gen_step_iter", "as_gen_step", 
-    "as_gen_step_iter", "split_cm, ""with_iter_next", 
-    "map", "filter", "reduce", "zip", "chunked", 
-    "foreach", "async_foreach", "through", "async_through", 
+    "as_gen_step_iter", "split_cm", "with_iter_next", 
+    "map", "filter", "reduce", "zip", "chain", 
+    "chain_from_iterable", "chunked", "foreach", 
+    "async_foreach", "through", "async_through", 
     "flatten", "async_flatten", "collect", "async_collect", 
     "group_collect", "async_group_collect", "iter_unique", 
     "async_iter_unique", "wrap_iter", "wrap_aiter", "acc_step", 
@@ -30,8 +31,8 @@ from contextlib import (
 )
 from copy import copy
 from dataclasses import dataclass
-from itertools import batched, pairwise
-from inspect import isawaitable, iscoroutinefunction, signature
+from itertools import batched, chain as _chain, pairwise
+from inspect import iscoroutinefunction, signature
 from sys import _getframe, exc_info
 from _thread import start_new_thread
 from time import sleep, time
@@ -42,7 +43,7 @@ from typing import (
 
 from asynctools import (
     async_filter, async_map, async_reduce, async_zip, async_batched, 
-    ensure_async, ensure_aiter, collect as async_collect, 
+    ensure_async, ensure_aiter, async_chain, collect as async_collect, 
 )
 from texttools import format_time
 from undefined import undefined
@@ -62,35 +63,32 @@ class YieldFrom:
     value: Any
 
 
-def iterable(obj, /) -> bool:
-    """判断对象是不是 Iterable
-    """
-    return isinstance(obj, Iterable)
+iterable = Iterable.__instancecheck__
+async_iterable = AsyncIterable.__instancecheck__
+isawaitable = Awaitable.__instancecheck__
 
 
-def async_iterable(obj, /) -> bool:
-    """判断对象是不是 AsyncIterable
-    """
-    return isinstance(obj, AsyncIterable)
+def _coalesce(vals, default=None):
+    for val in vals:
+        if val is not None:
+            return val
+    return default
 
 
-def _get_async(back: int = 2, /) -> bool:
+@overload
+def _get_async(back: int = 2, /, *, default: Literal[False] = False) -> bool:
+    ...
+@overload
+def _get_async[T](back: int = 2, /, *, default: T) -> bool | T:
+    ...
+def _get_async[T](back: int = 2, /, *, default: Literal[False] | T = False) -> bool | T:
     """往上查找，从最近的调用栈的命名空间中获取 `async_` 的值
     """
-    f: None | FrameType
-    f = _getframe(back)
-    f_globals = f.f_globals
-    f_locals  = f.f_locals
-    if f_locals is f_globals:
-        return f_locals.get("async_") or False
-    while f_locals is not None and f_locals is not f_globals:
-        if "async_" in f_locals:
-            return f_locals["async_"] or False
-        f = f.f_back
-        if f is None:
-            break
-        f_locals = f.f_locals
-    return False
+    def iter_frams(f: None | FrameType = _getframe(back)):
+        while f:
+            yield f.f_locals.get("async_")
+            f = f.f_back
+    return _coalesce(iter_frams(), default)
 
 
 def _run_gen_step(gen: Generator, /):
@@ -109,14 +107,17 @@ async def _run_gen_step_async(gen: Generator, /):
     send  = gen.send
     throw = gen.throw
     try:
-        ret: Awaitable = send(None)
+        ret: Any = send(None)
         while True:
-            try:
-                value: Any = await ret
-            except BaseException as e:
-                ret = throw(e)
+            if isawaitable(ret):
+                try:
+                    value: Any = (await ret)
+                except BaseException as e:
+                    ret = throw(e)
+                else:
+                    ret = send(value)
             else:
-                ret = send(value)
+                ret = send(ret)
     except StopIteration as e:
         if isawaitable(e.value):
             return await e.value
@@ -173,10 +174,10 @@ async def _run_gen_step_async_iter(gen: Generator, /) -> AsyncIterator:
     send  = gen.send
     throw = gen.throw
     try:
-        ret: Awaitable | Yield | YieldFrom = send(None)
+        ret: Any = send(None)
         while True:
             try:
-                if isinstance(ret, Awaitable):
+                if isawaitable(ret):
                     value: Any = await ret
                 else:
                     value = ret.value
@@ -269,9 +270,17 @@ def as_gen_step[**Args](
     gen_step: Callable[Args, Generator], 
     /, 
 ) -> Callable[Args, Any]:
+    default_async = _get_async(default=None)
     def wrapper(*args: Args.args, **kwds: Args.kwargs):
-        async_ = cast(None | Literal[False, True], kwds.pop("async_", None))
-        return run_gen_step(gen_step, async_, *args, **kwds)
+        return run_gen_step(
+            gen_step, 
+            _coalesce((
+                kwds.pop("async_", None), 
+                _get_async(default=default_async), 
+            )), 
+            *args, 
+            **kwds, 
+        )
     return wrapper
 
 
@@ -279,9 +288,17 @@ def as_gen_step_iter[**Args](
     gen_step: Callable[Args, Generator], 
     /, 
 ) -> Callable[Args, Iterable | AsyncIterable]:
+    default_async = _get_async(default=None)
     def wrapper(*args: Args.args, **kwds: Args.kwargs):
-        async_ = cast(None | Literal[False, True], kwds.pop("async_", None))
-        return run_gen_step_iter(gen_step, async_, *args, **kwds)
+        return run_gen_step_iter(
+            gen_step, 
+            _coalesce((
+                kwds.pop("async_", None), 
+                _get_async(default=default_async), 
+            )), 
+            *args, 
+            **kwds, 
+        )
     return wrapper
 
 
@@ -329,7 +346,7 @@ def split_cm[T](
             finally:
                 yield exit()
 
-        run_gen_step(gen_step, async_=async_)
+        run_gen_step(gen_step, async_)
     """
     if isinstance(cm, AbstractAsyncContextManager):
         enter: Callable = cm.__aenter__
@@ -380,7 +397,7 @@ def with_iter_next[T](
                     e = yield do_next()
                     do_what_you_want()
 
-        run_gen_step(gen_step, async_=async_)
+        run_gen_step(gen_step, async_)
     """
     if isinstance(iterable, AsyncIterable):
         try:
@@ -399,17 +416,21 @@ def map(
     iterable: Iterable | AsyncIterable, 
     /, 
     *iterables: Iterable | AsyncIterable, 
+    threaded: bool = False, 
 ):
     """
     """
     if (
+        threaded or
         iscoroutinefunction(function) or 
         isinstance(iterable, AsyncIterable) or 
         any(isinstance(i, AsyncIterable) for i in iterables)
     ):
         if function is None:
             if iterables:
-                return async_zip(iterable, *iterables)
+                return async_zip(iterable, *iterables, threaded=threaded)
+            elif threaded:
+                return ensure_aiter(iterable, threaded=threaded)
             else:
                 return iterable
         return async_map(function, iterable, *iterables)
@@ -425,11 +446,12 @@ def filter(
     function: None | Callable, 
     iterable: Iterable | AsyncIterable, 
     /, 
+    threaded: bool = False, 
 ):
     """
     """
-    if iscoroutinefunction(function) or isinstance(iterable, AsyncIterable):
-        return async_filter(function, iterable)
+    if threaded or iscoroutinefunction(function) or isinstance(iterable, AsyncIterable):
+        return async_filter(function, iterable, threaded=threaded)
     return _filter(function, iterable)
 
 
@@ -438,11 +460,12 @@ def reduce(
     iterable: Iterable | AsyncIterable, 
     initial: Any = undefined, 
     /, 
+    threaded: bool = False, 
 ):
     """
     """
-    if iscoroutinefunction(function) or isinstance(iterable, AsyncIterable):
-        return async_reduce(function, iterable, initial)
+    if threaded or iscoroutinefunction(function) or isinstance(iterable, AsyncIterable):
+        return async_reduce(function, iterable, initial, threaded=threaded)
     from functools import reduce
     if initial is undefined:
         return reduce(function, iterable)
@@ -453,12 +476,78 @@ def zip(
     iterable: Iterable | AsyncIterable, 
     /, 
     *iterables: Iterable | AsyncIterable, 
+    threaded: bool = False, 
 ):
     """
     """
-    if isinstance(iterable, AsyncIterable) or any(isinstance(i, AsyncIterable) for i in iterables):
-        return async_zip(iterable, *iterables)
-    return _zip(iterable, *iterables)
+    if (not threaded and 
+        isinstance(iterable, Iterable) and 
+        all(isinstance(it, Iterable) for it in iterables)
+    ):
+        return _zip(iterable, *iterables)
+    return async_zip(iterable, *iterables, threaded=threaded)
+
+
+@overload
+def chain[T](
+    iterable: Iterable[T], 
+    /, 
+    *iterables: Iterable[T], 
+    threaded: bool = False, 
+) -> Iterator[T]:
+    ...
+@overload
+def chain[T](
+    iterable: AsyncIterable[T], 
+    /, 
+    *iterables: Iterable[T] | AsyncIterable[T], 
+    threaded: bool = False, 
+) -> AsyncIterator[T]:
+    ...
+def chain[T](
+    iterable: Iterable[T] | AsyncIterable[T], 
+    /, 
+    *iterables: Iterable[T] | AsyncIterable[T], 
+    threaded: bool = False, 
+) -> Iterator[T] | AsyncIterator[T]:
+    if (not threaded and 
+        isinstance(iterable, Iterable) and 
+        all(isinstance(it, Iterable) for it in iterables)
+    ):
+        return _chain(iterable, *iterables) # type: ignore
+    return async_chain(iterable, *iterables, threaded=threaded)
+
+
+@overload
+def chain_from_iterable[T](
+    iterables: Iterable[Iterable[T]], 
+    threaded: Literal[False] = False, 
+    *, 
+    async_: Literal[False] = False, 
+) -> Iterator[T]:
+    ...
+@overload
+def chain_from_iterable[T](
+    iterables: Iterable[Iterable[T] | AsyncIterable[T]] | AsyncIterable[Iterable[T] | AsyncIterable[T]], 
+    threaded: bool = False, 
+    *, 
+    async_: Literal[True], 
+) -> AsyncIterator[T]:
+    ...
+def chain_from_iterable[T](
+    iterables: Iterable[Iterable[T]] | Iterable[Iterable[T] | AsyncIterable[T]] | AsyncIterable[Iterable[T] | AsyncIterable[T]], 
+    threaded: bool = False, 
+    *, 
+    async_: Literal[False, True] = False, 
+) -> Iterator[T] | AsyncIterator[T]:
+    if async_ or threaded or not isinstance(iterables, Iterable):
+        if isinstance(iterables, Iterable):
+            return async_chain.from_iterable(iterables, threaded=threaded)
+        else:
+            return async_chain.from_iterable(iterables, threaded=threaded)
+    return _chain.from_iterable(iterables) # type: ignore
+
+setattr(chain, "from_iterable", chain_from_iterable)
 
 
 @overload
@@ -466,6 +555,7 @@ def chunked[T](
     iterable: Iterable[T], 
     n: int = 1, 
     /, 
+    threaded: bool = False, 
 ) -> Iterator[Sequence[T]]:
     ...
 @overload
@@ -473,12 +563,14 @@ def chunked[T](
     iterable: AsyncIterable[T], 
     n: int = 1, 
     /, 
+    threaded: bool = False, 
 ) -> AsyncIterator[Sequence[T]]:
     ...
 def chunked[T](
     iterable: Iterable[T] | AsyncIterable[T], 
     n: int = 1, 
     /, 
+    threaded: bool = False, 
 ) -> Iterator[Sequence[T]] | AsyncIterator[Sequence[T]]:
     """
     """
@@ -488,10 +580,10 @@ def chunked[T](
         if n == 1:
             return ((e,) for e in iterable)
         return (iterable[i:j] for i, j in pairwise(range(0, len(iterable)+n, n)))
-    elif isinstance(iterable, Iterable):
+    elif not threaded and isinstance(iterable, Iterable):
         return batched(iterable, n)
     else:
-        return async_batched(iterable, n)
+        return async_batched(iterable, n, threaded=threaded)
 
 
 def foreach(
@@ -499,17 +591,21 @@ def foreach(
     iterable: Iterable | AsyncIterable, 
     /, 
     *iterables: Iterable | AsyncIterable, 
+    threaded: bool = False, 
 ):
     """
     """
-    if not (isinstance(iterable, Iterable) and all(isinstance(it, Iterable) for it in iterables)):
-        return async_foreach(value, iterable, *iterables)
-    if iterables:
-        for args in _zip(iterable, *iterables):
-            value(*args)
-    else:
-        for arg in iterable:
-            value(arg)
+    if (not threaded and 
+        isinstance(iterable, Iterable) and 
+        all(isinstance(it, Iterable) for it in iterables)
+    ):
+        if iterables:
+            for args in _zip(iterable, *iterables):
+                value(*args)
+        else:
+            for arg in iterable:
+                value(arg)
+    return async_foreach(value, iterable, *iterables, threaded=threaded)
 
 
 async def async_foreach(
@@ -534,11 +630,12 @@ def through(
     iterable: Iterable | AsyncIterable, 
     /, 
     take_while: None | Callable = None, 
+    threaded: bool = False, 
 ):
     """
     """
-    if not isinstance(iterable, Iterable):
-        return async_through(iterable, take_while)
+    if threaded or not isinstance(iterable, Iterable):
+        return async_through(iterable, take_while, threaded=threaded)
     if take_while is None:
         for _ in iterable:
             pass
@@ -575,6 +672,7 @@ def flatten(
     iterable: Iterable, 
     /, 
     exclude_types: type | tuple[type, ...] = (Buffer, str), 
+    threaded: bool = False, 
 ) -> Iterator:
     ...
 @overload
@@ -582,17 +680,19 @@ def flatten(
     iterable: AsyncIterable, 
     /, 
     exclude_types: type | tuple[type, ...] = (Buffer, str), 
+    threaded: bool = False, 
 ) -> AsyncIterator:
     ...
 def flatten(
     iterable: Iterable | AsyncIterable, 
     /, 
     exclude_types: type | tuple[type, ...] = (Buffer, str), 
+    threaded: bool = False, 
 ) -> Iterator | AsyncIterator:
     """
     """
-    if not isinstance(iterable, Iterable):
-        return async_flatten(iterable, exclude_types)
+    if threaded or not isinstance(iterable, Iterable):
+        return async_flatten(iterable, exclude_types, threaded=threaded)
     def gen(iterable):
         for e in iterable:
             if isinstance(e, (Iterable, AsyncIterable)) and not isinstance(e, exclude_types):
@@ -623,6 +723,7 @@ def collect[K, V](
     iterable: Iterable[tuple[K, V]] | Mapping[K, V], 
     /, 
     rettype: Callable[[Iterable[tuple[K, V]]], MutableMapping[K, V]], 
+    threaded: bool = False, 
 ) -> MutableMapping[K, V]:
     ...
 @overload
@@ -630,6 +731,7 @@ def collect[T](
     iterable: Iterable[T], 
     /, 
     rettype: Callable[[Iterable[T]], Collection[T]] = list, 
+    threaded: bool = False, 
 ) -> Collection[T]:
     ...
 @overload
@@ -637,6 +739,7 @@ def collect[K, V](
     iterable: AsyncIterable[tuple[K, V]], 
     /, 
     rettype: Callable[[Iterable[tuple[K, V]]], MutableMapping[K, V]], 
+    threaded: bool = False, 
 ) -> Coroutine[Any, Any, MutableMapping[K, V]]:
     ...
 @overload
@@ -644,17 +747,19 @@ def collect[T](
     iterable: AsyncIterable[T], 
     /, 
     rettype: Callable[[Iterable[T]], Collection[T]] = list, 
+    threaded: bool = False, 
 ) -> Coroutine[Any, Any, Collection[T]]:
     ...
 def collect(
     iterable: Iterable | AsyncIterable | Mapping, 
     /, 
     rettype: Callable[[Iterable], Collection] = list, 
+    threaded: bool = False, 
 ) -> Collection | Coroutine[Any, Any, Collection]:
     """
     """
-    if not isinstance(iterable, Iterable):
-        return async_collect(iterable, rettype)
+    if threaded or not isinstance(iterable, Iterable):
+        return async_collect(iterable, rettype, threaded=threaded)
     return rettype(iterable)
 
 
@@ -663,6 +768,7 @@ def group_collect[K, V, C: Container](
     iterable: Iterable[tuple[K, V]], 
     mapping: None = None, 
     factory: None | C | Callable[[], C] = None, 
+    threaded: bool = False, 
 ) -> dict[K, C]:
     ...
 @overload
@@ -670,6 +776,7 @@ def group_collect[K, V, C: Container, M: MutableMapping](
     iterable: Iterable[tuple[K, V]], 
     mapping: M, 
     factory: None | C | Callable[[], C] = None, 
+    threaded: bool = False, 
 ) -> M:
     ...
 @overload
@@ -677,6 +784,7 @@ def group_collect[K, V, C: Container](
     iterable: AsyncIterable[tuple[K, V]], 
     mapping: None = None, 
     factory: None | C | Callable[[], C] = None, 
+    threaded: bool = False, 
 ) -> Coroutine[Any, Any, dict[K, C]]:
     ...
 @overload
@@ -684,17 +792,19 @@ def group_collect[K, V, C: Container, M: MutableMapping](
     iterable: AsyncIterable[tuple[K, V]], 
     mapping: M, 
     factory: None | C | Callable[[], C] = None, 
+    threaded: bool = False, 
 ) -> Coroutine[Any, Any, M]:
     ...
 def group_collect[K, V, C: Container, M: MutableMapping](
     iterable: Iterable[tuple[K, V]] | AsyncIterable[tuple[K, V]], 
     mapping: None | M = None, 
     factory: None | C | Callable[[], C] = None, 
+    threaded: bool = False, 
 ) -> dict[K, C] | M | Coroutine[Any, Any, dict[K, C]] | Coroutine[Any, Any, M]:
     """
     """
-    if not isinstance(iterable, Iterable):
-        return async_group_collect(iterable, mapping, factory)
+    if threaded or not isinstance(iterable, Iterable):
+        return async_group_collect(iterable, mapping, factory, threaded=threaded)
     if factory is None:
         if isinstance(mapping, defaultdict):
             factory = mapping.default_factory
@@ -791,6 +901,7 @@ def iter_unique[T](
     iterable: Iterable[T], 
     /, 
     seen: None | MutableSet = None, 
+    threaded: bool = False, 
 ) -> Iterator[T]:
     ...
 @overload
@@ -798,17 +909,19 @@ def iter_unique[T](
     iterable: AsyncIterable[T], 
     /, 
     seen: None | MutableSet = None, 
+    threaded: bool = False, 
 ) -> AsyncIterator[T]:
     ...
 def iter_unique[T](
     iterable: Iterable[T] | AsyncIterable[T], 
     /, 
     seen: None | MutableSet = None, 
+    threaded: bool = False, 
 ) -> Iterator[T] | AsyncIterator[T]:
     """
     """
-    if not isinstance(iterable, Iterable):
-        return async_iter_unique(iterable, seen)
+    if threaded or not isinstance(iterable, Iterable):
+        return async_iter_unique(iterable, seen, threaded=threaded)
     if seen is None:
         seen = set()
     def gen(iterable):
@@ -1016,7 +1129,7 @@ def context[T](
                             add_arg(enter(ctx))
                     ret = func(*args)
                     if isawaitable(ret):
-                        ret = await ret
+                        ret = await cast(Awaitable, ret)
                     return ret
         return call()
     else:
